@@ -1,5 +1,5 @@
 //  gnoMint: a graphical interface for managing a certification authority
-//  Copyright (C) 2006,2007,2008 David Marín Carreño <davefx@gmail.com>
+//  Copyright (C) 2006-2009 David Marín Carreño <davefx@gmail.com>
 //
 //  This file is part of gnoMint.
 //
@@ -32,21 +32,24 @@
 
 
 #include "ca.h"
-#include "ca_policy.h"
 #include "ca_file.h"
 #include "certificate_properties.h"
 #include "crl.h"
 #include "csr_properties.h"
-#include "tls.h"
+#include "dialog.h"
+#include "export.h"
 #include "new_ca_window.h"
 #include "new_req_window.h"
-#include "new_cert_window.h"
-#include "pkey_manage.h"
+#include "new_cert.h"
 #include "preferences-gui.h"
+#include "preferences-window.h"
 #include "import.h"
 
 #ifndef GNOMINTCLI
 
+#define GNOMINT_MIME_TYPE "application/x-gnomint"
+
+
 enum {CA_MODEL_COLUMN_ID=0,
       CA_MODEL_COLUMN_IS_CA=1,
       CA_MODEL_COLUMN_SERIAL=2,
@@ -79,6 +82,7 @@ extern GladeXML * main_window_xml;
 extern GladeXML * cert_popup_menu_xml;
 extern GladeXML * csr_popup_menu_xml;
 
+
 static GtkTreeStore * ca_model = NULL;
 static gboolean cert_title_inserted = FALSE;
 static GtkTreeIter * cert_parent_iter = NULL;
@@ -151,7 +155,7 @@ int __ca_refresh_model_add_certificate (void *pArg, int argc, char **argv, char
 			gtk_tree_iter_free (last_parent_iter);
 		last_parent_iter = NULL;
 	} else {
-		// If not, then we must find the parent of the current nod
+		// If not, then we must find the parent of the current node
 		gtk_tree_model_get_value (GTK_TREE_MODEL(new_model), last_cert_iter, CA_MODEL_COLUMN_DN, last_dn_value);
 		gtk_tree_model_get_value (GTK_TREE_MODEL(new_model), last_cert_iter, CA_MODEL_COLUMN_PARENT_DN, 
 					  last_parent_dn_value);
@@ -378,7 +382,7 @@ void __ca_tree_view_private_key_in_db_datafunc (GtkTreeViewColumn *tree_column,
 
 
 
-gboolean ca_refresh_model () 
+gboolean ca_refresh_model_callback () 
 {
 	GtkTreeStore * new_model = NULL;
 	GtkTreeView * treeview = NULL;
@@ -622,19 +626,31 @@ void __ca_activate_certificate_selection (GtkTreeIter *iter)
 			   CA_MODEL_COLUMN_IS_CA, &is_ca, 
 			   CA_MODEL_COLUMN_PRIVATE_KEY_IN_DB, &pk_indb, 
 			   CA_MODEL_COLUMN_REVOCATION, &is_revoked, -1);
-	if (pk_indb) {
-		widget = glade_xml_get_widget (main_window_xml, "extractprivatekey1");
-		gtk_widget_set_sensitive (widget, TRUE);
-	}
+
+	widget = glade_xml_get_widget (main_window_xml, "extractprivatekey1");
+	gtk_widget_set_sensitive (widget, pk_indb);
+	widget = glade_xml_get_widget (main_window_xml, "extractpkey_toolbutton");
+	gtk_widget_set_sensitive (widget, pk_indb);
 
         widget = glade_xml_get_widget (main_window_xml, "revoke1");
         gtk_widget_set_sensitive (widget, (! is_revoked));
+        widget = glade_xml_get_widget (main_window_xml, "revoke_toolbutton");
+        gtk_widget_set_sensitive (widget, (! is_revoked));
+
 
 	widget = glade_xml_get_widget (main_window_xml, "sign1");
 	gtk_widget_set_sensitive (widget, FALSE);
+	widget = glade_xml_get_widget (main_window_xml, "sign_toolbutton");
+	gtk_widget_set_sensitive (widget, FALSE);
 
 	widget = glade_xml_get_widget (main_window_xml, "delete2");
 	gtk_widget_set_sensitive (widget, FALSE);
+	widget = glade_xml_get_widget (main_window_xml, "delete_toolbutton");
+	gtk_widget_set_sensitive (widget, FALSE);
+
+	widget = glade_xml_get_widget (main_window_xml, "properties1");
+	gtk_widget_set_sensitive (widget, TRUE);
+
 
 }
 
@@ -647,19 +663,29 @@ void __ca_activate_csr_selection (GtkTreeIter *iter)
 	gtk_widget_set_sensitive (widget, TRUE);
 
 	gtk_tree_model_get(GTK_TREE_MODEL(ca_model), iter, CA_MODEL_COLUMN_PRIVATE_KEY_IN_DB, &pk_indb, -1);
-	if (pk_indb) {
-		widget = glade_xml_get_widget (main_window_xml, "extractprivatekey1");
-		gtk_widget_set_sensitive (widget, TRUE);
-	}
+
+	widget = glade_xml_get_widget (main_window_xml, "extractprivatekey1");
+	gtk_widget_set_sensitive (widget, pk_indb);
+	widget = glade_xml_get_widget (main_window_xml, "extractpkey_toolbutton");
+	gtk_widget_set_sensitive (widget, pk_indb);
 
 	widget = glade_xml_get_widget (main_window_xml, "revoke1");
 	gtk_widget_set_sensitive (widget, FALSE);
+	widget = glade_xml_get_widget (main_window_xml, "revoke_toolbutton");
+	gtk_widget_set_sensitive (widget, FALSE);
 
 	widget = glade_xml_get_widget (main_window_xml, "sign1");
 	gtk_widget_set_sensitive (widget, TRUE);
+	widget = glade_xml_get_widget (main_window_xml, "sign_toolbutton");
+	gtk_widget_set_sensitive (widget, TRUE);
 
 	widget = glade_xml_get_widget (main_window_xml, "delete2");
 	gtk_widget_set_sensitive (widget, TRUE);
+	widget = glade_xml_get_widget (main_window_xml, "delete_toolbutton");
+	gtk_widget_set_sensitive (widget, TRUE);
+
+	widget = glade_xml_get_widget (main_window_xml, "properties1");
+	gtk_widget_set_sensitive (widget, TRUE);
 }
 
 void __ca_deactivate_actions ()
@@ -671,15 +697,26 @@ void __ca_deactivate_actions ()
 
 	widget = glade_xml_get_widget (main_window_xml, "extractprivatekey1");
 	gtk_widget_set_sensitive (widget, FALSE);
+	widget = glade_xml_get_widget (main_window_xml, "extractpkey_toolbutton");
+	gtk_widget_set_sensitive (widget, FALSE);
 
 	widget = glade_xml_get_widget (main_window_xml, "revoke1");
 	gtk_widget_set_sensitive (widget, FALSE);
+	widget = glade_xml_get_widget (main_window_xml, "revoke_toolbutton");
+	gtk_widget_set_sensitive (widget, FALSE);
 
 	widget = glade_xml_get_widget (main_window_xml, "sign1");
 	gtk_widget_set_sensitive (widget, FALSE);
+	widget = glade_xml_get_widget (main_window_xml, "sign_toolbutton");
+	gtk_widget_set_sensitive (widget, FALSE);
 
 	widget = glade_xml_get_widget (main_window_xml, "delete2");
 	gtk_widget_set_sensitive (widget, FALSE);
+	widget = glade_xml_get_widget (main_window_xml, "delete_toolbutton");
+	gtk_widget_set_sensitive (widget, FALSE);
+
+	widget = glade_xml_get_widget (main_window_xml, "properties1");
+	gtk_widget_set_sensitive (widget, FALSE);
 }
 
 gint __ca_selection_type (GtkTreeView *tree_view, GtkTreeIter **iter) {
@@ -690,7 +727,7 @@ gint __ca_selection_type (GtkTreeView *tree_view, GtkTreeIter **iter) {
 	GtkTreePath *selection_path = NULL;
 
 	if (gtk_tree_selection_count_selected_rows (selection) != 1)
-		return 0;
+		return -1;
 
 	gtk_tree_selection_get_selected (selection, NULL, &selection_iter); 
 	if (iter)
@@ -702,7 +739,7 @@ gint __ca_selection_type (GtkTreeView *tree_view, GtkTreeIter **iter) {
 	if (gtk_tree_path_is_ancestor (parent, selection_path) && gtk_tree_path_compare (parent, selection_path)) {
 		gtk_tree_path_free (parent);
 		/* It's a certificate */
-		return 1;
+		return CA_FILE_ELEMENT_TYPE_CERT;
 	}
 
 	gtk_tree_path_free (parent);
@@ -710,11 +747,11 @@ gint __ca_selection_type (GtkTreeView *tree_view, GtkTreeIter **iter) {
 	if (gtk_tree_path_is_ancestor (parent, selection_path) && gtk_tree_path_compare (parent, selection_path)) {
 		gtk_tree_path_free (parent);
 		/* It's a CSR */
-		return 2;
+		return CA_FILE_ELEMENT_TYPE_CSR;
 	}
 
 	gtk_tree_path_free (parent);
-	return 0;
+	return -1;
 }
 
 gboolean ca_treeview_selection_change (GtkTreeView *tree_view,
@@ -722,13 +759,13 @@ gboolean ca_treeview_selection_change (GtkTreeView *tree_view,
 {
 	GtkTreeIter *selection_iter;
 	switch (__ca_selection_type (tree_view, &selection_iter)) {
-	case 1:
+	case CA_FILE_ELEMENT_TYPE_CERT:
 		__ca_activate_certificate_selection (selection_iter);
 		break;
-	case 2:
+	case CA_FILE_ELEMENT_TYPE_CSR:
 		__ca_activate_csr_selection (selection_iter);
 		break;
-	case 0:
+	case -1:
 	default:
 		__ca_deactivate_actions();
 		break;
@@ -737,25 +774,6 @@ gboolean ca_treeview_selection_change (GtkTreeView *tree_view,
 	return FALSE;
 }
 
-void ca_error_dialog (gchar *message) {
-        GtkWidget *dialog, *widget;
-   
-        widget = glade_xml_get_widget (main_window_xml, "main_window1");
-   
-        /* Create the widgets */
-   
-        dialog = gtk_message_dialog_new (GTK_WINDOW(widget),
-                                         GTK_DIALOG_DESTROY_WITH_PARENT,
-                                         GTK_MESSAGE_ERROR,
-                                         GTK_BUTTONS_CLOSE,
-                                         "%s",
-                                         message);
-   
-        gtk_dialog_run (GTK_DIALOG(dialog));
-   
-        gtk_widget_destroy (dialog);
-
-}
 
 void __ca_export_public_pem (GtkTreeIter *iter, gint type)
 {
@@ -792,9 +810,9 @@ void __ca_export_public_pem (GtkTreeIter *iter, gint type)
 		if (error) {
 			gtk_widget_destroy (GTK_WIDGET(dialog));
 			if (type == 1)
-				ca_error_dialog (_("There was an error while exporting certificate."));
+				dialog_error (_("There was an error while exporting certificate."));
 			else
-				ca_error_dialog (_("There was an error while exporting CSR."));
+				dialog_error (_("There was an error while exporting CSR."));
 			return;
 		} 
 
@@ -827,27 +845,27 @@ void __ca_export_public_pem (GtkTreeIter *iter, gint type)
 
                 if (error) {
                         gtk_widget_destroy (GTK_WIDGET(dialog));
-                        if (type == 1)
-                                ca_error_dialog (_("There was an error while exporting certificate."));
+                        if (type == CA_FILE_ELEMENT_TYPE_CERT)
+                                dialog_error (_("There was an error while exporting certificate."));
                         else
-                                ca_error_dialog (_("There was an error while exporting CSR."));
+                                dialog_error (_("There was an error while exporting CSR."));
                         return;
                 } 
 
                 g_io_channel_shutdown (file, TRUE, &error);
                 if (error) {
                         gtk_widget_destroy (GTK_WIDGET(dialog));
-                        if (type == 1)
-                                ca_error_dialog (_("There was an error while exporting certificate."));
+                        if (type == CA_FILE_ELEMENT_TYPE_CERT)
+                                dialog_error (_("There was an error while exporting certificate."));
                         else
-                                ca_error_dialog (_("There was an error while exporting CSR."));
+                                dialog_error (_("There was an error while exporting CSR."));
                         return;
                 } 
 
                 g_io_channel_unref (file);
 
                 gtk_widget_destroy (GTK_WIDGET(dialog));
-                if (type == 1)
+                if (type == CA_FILE_ELEMENT_TYPE_CERT)
                         dialog = GTK_DIALOG(gtk_message_dialog_new (GTK_WINDOW(widget),
                                                                     GTK_DIALOG_DESTROY_WITH_PARENT,
                                                                     GTK_MESSAGE_INFO,
@@ -869,84 +887,6 @@ void __ca_export_public_pem (GtkTreeIter *iter, gint type)
 }
 
 
-gchar * ca_dialog_get_password (gchar *info_message, 
-                                gchar *password_message, gchar *confirm_message, 
-                                gchar *distinct_error_message, guint minimum_length)
-{
-	GtkWidget * widget = NULL, * password_widget = NULL;
-	//GtkDialog * dialog = NULL;
-	GladeXML * dialog_xml = NULL;
-	gchar     * xml_file = NULL;
-	gint response = 0;
-	gchar *password = NULL;
-	const gchar *passwordagain = NULL;
-
-	xml_file = g_build_filename (PACKAGE_DATA_DIR, "gnomint", "gnomint.glade", NULL );
-	dialog_xml = glade_xml_new (xml_file, "get_password_dialog", NULL);
-	g_free (xml_file);
-	glade_xml_signal_autoconnect (dialog_xml); 	
-	
-	widget = glade_xml_get_widget (dialog_xml, "info_message");
-	gtk_label_set_text (GTK_LABEL(widget), info_message);
-	widget = glade_xml_get_widget (dialog_xml, "password_message");
-	gtk_label_set_text (GTK_LABEL(widget), password_message);
-	widget = glade_xml_get_widget (dialog_xml, "confirm_message");
-	gtk_label_set_text (GTK_LABEL(widget), confirm_message);
-
-	password_widget = glade_xml_get_widget (dialog_xml, "password_entry");
-	widget = glade_xml_get_widget (dialog_xml, "password_dialog_ok_button");
-	g_object_set_data (G_OBJECT(password_widget), "minimum_length", 
-                           GINT_TO_POINTER(minimum_length));
-	g_object_set_data (G_OBJECT(password_widget), "ok_button", widget);
-
-	do {
-		gtk_widget_grab_focus (password_widget);
-
-		if (password)
-			g_free (password);
-
-		widget = glade_xml_get_widget (dialog_xml, "get_password_dialog");
-		response = gtk_dialog_run(GTK_DIALOG(widget)); 
-	
-		if (!response) {
-			gtk_widget_destroy (widget);
-			g_object_unref (G_OBJECT(dialog_xml));
-			return NULL;
-		} else {
-			widget = glade_xml_get_widget (dialog_xml, "password_entry");
-			password = g_strdup(gtk_entry_get_text (GTK_ENTRY(widget)));
-			widget = glade_xml_get_widget (dialog_xml, "confirm_entry");
-			passwordagain = gtk_entry_get_text (GTK_ENTRY(widget));
-		}
-		
-		if (strcmp (password, passwordagain)) {
-			ca_error_dialog (distinct_error_message);
-		}
-
-	} while (strcmp (password, passwordagain));
-
-	widget = glade_xml_get_widget (dialog_xml, "get_password_dialog");
-	gtk_widget_destroy (widget);
-	g_object_unref (G_OBJECT(dialog_xml));
-	
-	return password;
-}
-
-void ca_password_entry_changed_cb (GtkEditable *password_entry, gpointer user_data)
-{
-	GtkWidget * button = GTK_WIDGET(g_object_get_data (G_OBJECT(password_entry), "ok_button"));
-	guint minimum_length = GPOINTER_TO_INT (g_object_get_data (G_OBJECT(password_entry), 
-                                                                   "minimum_length"));
-
-	if (strlen (gtk_entry_get_text (GTK_ENTRY(password_entry))) >= minimum_length)
-		gtk_widget_set_sensitive (button, TRUE);
-	else
-		gtk_widget_set_sensitive (button, FALSE);
-	
-}
-
-
-
 gchar * __ca_export_private_pkcs8 (GtkTreeIter *iter, gint type)
 {
 	GtkWidget *widget = NULL;
@@ -977,7 +917,7 @@ gchar * __ca_export_private_pkcs8 (GtkTreeIter *iter, gint type)
 	
 	gtk_tree_model_get(GTK_TREE_MODEL(ca_model), iter, CA_MODEL_COLUMN_ID, &id, -1);		
 	
-	strerror = ca_export_private_pkcs8 (id, type, filename);
+	strerror = export_private_pkcs8 (id, type, filename);
 
 	if (! strerror) {
 	
@@ -991,7 +931,7 @@ gchar * __ca_export_private_pkcs8 (GtkTreeIter *iter, gint type)
 		
 		gtk_widget_destroy (GTK_WIDGET(dialog));
 	} else {
-		ca_error_dialog (strerror);
+		dialog_error (strerror);
 	}
 
 	return filename;
@@ -1001,14 +941,10 @@ gchar * __ca_export_private_pkcs8 (GtkTreeIter *iter, gint type)
 void __ca_export_private_pem (GtkTreeIter *iter, gint type)
 {
 	GtkWidget *widget = NULL;
-	GIOChannel * file = NULL;
 	gchar * filename = NULL;
 	GtkDialog * dialog = NULL;
-	GError * error = NULL;
 	gint id;
-	gchar * pem = NULL;
-	PkeyManageData * crypted_pkey = NULL;
-	gchar * dn = NULL;
+        gchar * error_msg = NULL;
 
 	widget = glade_xml_get_widget (main_window_xml, "main_window1");
 	
@@ -1018,78 +954,35 @@ void __ca_export_private_pem (GtkTreeIter *iter, gint type)
 							  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 							  GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 							  NULL));
-		
+        
 	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
 	
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_ACCEPT) {
 		gtk_widget_destroy (GTK_WIDGET(dialog));
 		return;
 	}
-
+        
 	filename = g_strdup(gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog)));
 	gtk_widget_destroy (GTK_WIDGET(dialog));
-	
-	file = g_io_channel_new_file (filename, "w", &error);
-	g_free (filename);
-	if (error) {
-		ca_error_dialog (_("There was an error while exporting private key."));
-		return;
-	} 
-	
-	gtk_tree_model_get(GTK_TREE_MODEL(ca_model), iter, CA_MODEL_COLUMN_ID, &id, -1);		
-	if (type == 1) {
-		crypted_pkey = pkey_manage_get_certificate_pkey (id);
-		dn = ca_file_get_dn_from_id (CA_FILE_ELEMENT_TYPE_CERT, id);
-	} else {
-		crypted_pkey = pkey_manage_get_csr_pkey (id);
-		dn = ca_file_get_dn_from_id (CA_FILE_ELEMENT_TYPE_CSR, id);
-	}
-	
-	if (!crypted_pkey || !dn) {
-		pkey_manage_data_free(crypted_pkey);
-		g_free (dn);
-		ca_error_dialog (_("There was an error while getting private key."));
-		return;
-	}
-	
-	pem =  pkey_manage_uncrypt (crypted_pkey, dn);
 
-	pkey_manage_data_free (crypted_pkey);
-	g_free (dn);
-	
-	if (!pem) {
-		ca_error_dialog (_("There was an error while decrypting private key."));
-		return;
-	}
-	
-	g_io_channel_write_chars (file, pem, strlen(pem), NULL, &error);
-	if (error) {
-		ca_error_dialog (_("There was an error while exporting private key."));
-		return;
-	} 
-	g_free (pem);
-	
-	
-	g_io_channel_shutdown (file, TRUE, &error);
-	if (error) {
-		ca_error_dialog (_("There was an error while exporting private key."));
-		g_io_channel_unref (file);
-		return;
-	} 
-	
-	g_io_channel_unref (file);
-	
-	gtk_widget_destroy (GTK_WIDGET(dialog));
-	dialog = GTK_DIALOG(gtk_message_dialog_new (GTK_WINDOW(widget),
-						    GTK_DIALOG_DESTROY_WITH_PARENT,
-						    GTK_MESSAGE_INFO,
-						    GTK_BUTTONS_CLOSE,
-						    "%s",
-						    _("Private key exported successfully")));
-	gtk_dialog_run (GTK_DIALOG(dialog));
-	
-	gtk_widget_destroy (GTK_WIDGET(dialog));
+	gtk_tree_model_get(GTK_TREE_MODEL(ca_model), iter, CA_MODEL_COLUMN_ID, &id, -1);		
 	
+        error_msg = export_private_pem (id, type, filename);
+        g_free (filename);
+
+        if (error_msg) {
+                dialog_error (error_msg);
+        } else {
+                dialog = GTK_DIALOG(gtk_message_dialog_new (GTK_WINDOW(widget),
+                                                            GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                            GTK_MESSAGE_INFO,
+                                                            GTK_BUTTONS_CLOSE,
+                                                            "%s",
+                                                            _("Private key exported successfully")));
+                gtk_dialog_run (GTK_DIALOG(dialog));
+                
+                gtk_widget_destroy (GTK_WIDGET(dialog));
+        }
 	
 }
 
@@ -1097,17 +990,11 @@ void __ca_export_private_pem (GtkTreeIter *iter, gint type)
 void __ca_export_pkcs12 (GtkTreeIter *iter, gint type)
 {
 	GtkWidget *widget = NULL;
-	GIOChannel * file = NULL;
 	gchar * filename = NULL;
-	gchar * password = NULL;
 	GtkDialog * dialog = NULL;
-	GError * error = NULL;
 	gint id;
-	gchar * crt_pem = NULL;
-	PkeyManageData * crypted_pkey = NULL;
-	gchar * dn = NULL;
-	gchar * privatekey = NULL;
-        gnutls_datum_t * pkcs12_datum = NULL;
+
+        gchar *error_msg = NULL;
 
 	widget = glade_xml_get_widget (main_window_xml, "main_window1");
 	
@@ -1128,95 +1015,24 @@ void __ca_export_pkcs12 (GtkTreeIter *iter, gint type)
 
 	filename = g_strdup(gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog)));
 	gtk_widget_destroy (GTK_WIDGET(dialog));
-
-
 	gtk_tree_model_get(GTK_TREE_MODEL(ca_model), iter, CA_MODEL_COLUMN_ID, &id, -1);		
 
-	if (type == 1) {
-		crypted_pkey = pkey_manage_get_certificate_pkey (id);
-		dn = ca_file_get_dn_from_id (CA_FILE_ELEMENT_TYPE_CERT, id);
-		crt_pem = ca_file_get_public_pem_from_id (CA_FILE_ELEMENT_TYPE_CERT, id);
-	}
-		
-	
-	if (! crypted_pkey || ! dn || ! crt_pem) {
-		ca_error_dialog (_("There was an error while getting the certificate and private key from the internal database."));
-		g_free (filename);
-		pkey_manage_data_free (crypted_pkey);
-		g_free (dn);
-		g_free (crt_pem);
-		return;
-	}
-	
-	privatekey = pkey_manage_uncrypt (crypted_pkey, dn);
+        error_msg = export_pkcs12 (id, type, filename);
 
-	if (! privatekey) {
-		ca_error_dialog (_("There was an error while getting the certificate and private key from the internal database."));
-		g_free (filename);
-		pkey_manage_data_free (crypted_pkey);
-		g_free (dn);
-		g_free (crt_pem);
-		return;
-	}
+        g_free (filename);
 
-	password = ca_dialog_get_password (_("You need to supply a passphrase for protecting the exported certificate, "
-					     "so nobody else but authorized people can use it. This passphrase will be asked "
-					     "by any application that will import the certificate."),
-					   _("Insert passphrase (8 characters or more):"), _("Insert passphrase (confirm):"), 
-					   _("The introduced passphrases are distinct."), 8);
-	if (! password) {
-		g_free (filename);
-		pkey_manage_data_free (crypted_pkey);
-		g_free (dn);
-		g_free (crt_pem);
-		g_free (privatekey);
-		return;
-	}
-			
-	pkcs12_datum = tls_generate_pkcs12 (crt_pem, privatekey, password); 
-	g_free (password);
-	g_free (privatekey);
-	pkey_manage_data_free (crypted_pkey);
-	g_free (dn);
-	g_free (crt_pem);
-	
-	
-	if (!pkcs12_datum) {
-		ca_error_dialog (_("There was an error while generating the PKCS#12 package."));
-		return;
-	}
-	
-	file = g_io_channel_new_file (filename, "w", &error);
-	g_free (filename);
-	if (error) {
-		ca_error_dialog (_("There was an error while exporting certificate."));
-		return;
-	} 
+        if (error_msg && strlen(error_msg)) {
+                dialog_error (error_msg);
+                return;
+        } 
+        
+        if (error_msg) {
+                // Export cancelled by user
+                return;
+        }
 
-        g_io_channel_set_encoding (file, NULL, NULL);
 
-	g_io_channel_write_chars (file, (gchar *) pkcs12_datum->data, pkcs12_datum->size, NULL, &error);
-	if (error) {
-		ca_error_dialog (_("There was an error while exporting the certificate."));
-                g_free (pkcs12_datum->data);
-                g_free (pkcs12_datum);
-		return;
-	} 
-        g_free (pkcs12_datum->data);
-	g_free (pkcs12_datum);
-	
-	
-	g_io_channel_shutdown (file, TRUE, &error);
-	if (error) {
-		ca_error_dialog (_("There was an error while exporting the certificate."));
-		g_io_channel_unref (file);
-		return;
-	} 
-	
-	g_io_channel_unref (file);
-	
-	gtk_widget_destroy (GTK_WIDGET(dialog));
-	dialog = GTK_DIALOG(gtk_message_dialog_new (GTK_WINDOW(widget),
+        dialog = GTK_DIALOG(gtk_message_dialog_new (GTK_WINDOW(widget),
 						    GTK_DIALOG_DESTROY_WITH_PARENT,
 						    GTK_MESSAGE_INFO,
 						    GTK_BUTTONS_CLOSE,
@@ -1249,8 +1065,10 @@ void ca_on_export1_activate (GtkMenuItem *menuitem, gpointer user_data)
 	gtk_tree_model_get(GTK_TREE_MODEL(ca_model), iter, CA_MODEL_COLUMN_PRIVATE_KEY_IN_DB, &has_pk_in_db, -1);			
 	widget = glade_xml_get_widget (dialog_xml, "privatepart_radiobutton2");
 	gtk_widget_set_sensitive (widget, has_pk_in_db);
+	widget = glade_xml_get_widget (dialog_xml, "bothparts_radiobutton3");
+	gtk_widget_set_sensitive (widget, has_pk_in_db);
 
-	if (type == 2) {
+	if (type == CA_FILE_ELEMENT_TYPE_CSR) {
  	        widget = glade_xml_get_widget (dialog_xml, "export_certificate_dialog");
 		gtk_window_set_title (GTK_WINDOW(widget), _("Export CSR - gnoMint"));
 
@@ -1325,7 +1143,7 @@ void ca_on_export1_activate (GtkMenuItem *menuitem, gpointer user_data)
 	
 	gtk_widget_destroy (widget);
 	g_object_unref (G_OBJECT(dialog_xml));
-	ca_error_dialog (_("Unexpected error"));
+	dialog_error (_("Unexpected error"));
 }
 
 void ca_on_extractprivatekey1_activate (GtkMenuItem *menuitem, gpointer user_data)
@@ -1345,19 +1163,14 @@ void ca_on_extractprivatekey1_activate (GtkMenuItem *menuitem, gpointer user_dat
 		return;
 	}
 	
-	if (type == 1)
+	if (type == CA_FILE_ELEMENT_TYPE_CERT)
 		ca_file_mark_pkey_as_extracted_for_id (CA_FILE_ELEMENT_TYPE_CERT, filename, id);
 	else
 		ca_file_mark_pkey_as_extracted_for_id (CA_FILE_ELEMENT_TYPE_CSR, filename, id);
 
 	g_free (filename);
 
-	ca_refresh_model ();
-}
-
-void ca_todo_callback ()
-{
-	ca_error_dialog (_("To do. Feature not implemented yet."));
+	dialog_refresh_list();
 }
 
 
@@ -1371,7 +1184,7 @@ void ca_on_revoke_activate (GtkMenuItem *menuitem, gpointer user_data)
 	gint response = 0;
 	gint id = 0;
 
-	if (type == 2)
+	if (type == CA_FILE_ELEMENT_TYPE_CSR)
 		return;
 	
 	widget = glade_xml_get_widget (main_window_xml, "main_window1");
@@ -1392,11 +1205,11 @@ void ca_on_revoke_activate (GtkMenuItem *menuitem, gpointer user_data)
 	gtk_tree_model_get(GTK_TREE_MODEL(ca_model), iter, CA_MODEL_COLUMN_ID, &id, -1);			
         errmsg = ca_file_revoke_crt (id);
 	if (errmsg) {
-                ca_error_dialog (_(errmsg));
+                dialog_error (_(errmsg));
 
         }
 
-	ca_refresh_model ();
+	dialog_refresh_list();
   
 }
 
@@ -1410,7 +1223,7 @@ void ca_on_delete2_activate (GtkMenuItem *menuitem, gpointer user_data)
 	gint response = 0;
 	gint id = 0;
 
-	if (type != 2)
+	if (type != CA_FILE_ELEMENT_TYPE_CSR)
 		return;
 	
 	widget = glade_xml_get_widget (main_window_xml, "main_window1");
@@ -1431,7 +1244,7 @@ void ca_on_delete2_activate (GtkMenuItem *menuitem, gpointer user_data)
 	gtk_tree_model_get(GTK_TREE_MODEL(ca_model), iter, CA_MODEL_COLUMN_ID, &id, -1);			
 	ca_file_remove_csr (id);
 
-	ca_refresh_model ();
+	dialog_refresh_list();
 }
 
 void ca_on_sign1_activate (GtkMenuItem *menuitem, gpointer user_data)
@@ -1441,13 +1254,14 @@ void ca_on_sign1_activate (GtkMenuItem *menuitem, gpointer user_data)
 	gint type = __ca_selection_type (GTK_TREE_VIEW(glade_xml_get_widget (main_window_xml, "ca_treeview")), &iter);
 	gchar * csr_pem;
 	gchar * csr_parent_id;
+	guint64 csr_id;
 
-	if (type != 2)
+	if (type != CA_FILE_ELEMENT_TYPE_CSR)
 		return;
 		
-	gtk_tree_model_get(GTK_TREE_MODEL(ca_model), iter, CA_MODEL_COLUMN_PEM, &csr_pem, CA_MODEL_COLUMN_PARENT_ID, &csr_parent_id, -1);
+	gtk_tree_model_get(GTK_TREE_MODEL(ca_model), iter, CA_MODEL_COLUMN_ID, &csr_id, CA_MODEL_COLUMN_PEM, &csr_pem, CA_MODEL_COLUMN_PARENT_ID, &csr_parent_id, -1);
 
-	new_cert_window_display (csr_pem, csr_parent_id);
+	new_cert_window_display (csr_id, csr_pem, csr_parent_id);
 	
 	g_free (csr_pem);
         g_free (csr_parent_id);
@@ -1462,11 +1276,10 @@ gboolean ca_open (gchar *filename, gboolean create)
 
 	__enable_widget ("new_certificate1");
 	__enable_widget ("save_as1");
-	__enable_widget ("properties1");
 	__enable_widget ("preferences1");
 
 
-	ca_refresh_model ();
+	dialog_refresh_list();
 	
 	
 	return TRUE;
@@ -1502,7 +1315,7 @@ void ca_update_csr_view (gboolean new_value, gboolean refresh)
         view_csr = new_value;
         gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(glade_xml_get_widget(main_window_xml, "csr_view_menuitem")), new_value);
         if (refresh)
-                ca_refresh_model ();
+                dialog_refresh_list();
 }
 
 gboolean ca_csr_view_toggled (GtkCheckMenuItem *button, gpointer user_data)
@@ -1519,7 +1332,7 @@ void ca_update_revoked_view (gboolean new_value, gboolean refresh)
         view_rcrt = new_value;
         gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(glade_xml_get_widget(main_window_xml, "revoked_view_menuitem")), new_value);
         if (refresh)
-                ca_refresh_model ();
+                dialog_refresh_list();
 }
 
 gboolean ca_rcrt_view_toggled (GtkCheckMenuItem *button, gpointer user_data)
@@ -1551,7 +1364,7 @@ gboolean ca_treeview_popup_timeout_program_cb (gpointer data)
 	selection_type  = __ca_selection_type (tree_view, &iter);
 	switch (selection_type) {
 		
-	case 1:
+	case CA_FILE_ELEMENT_TYPE_CERT:
 		menu = glade_xml_get_widget (cert_popup_menu_xml,
 					     "certificate_popup_menu");
 		
@@ -1568,7 +1381,7 @@ gboolean ca_treeview_popup_timeout_program_cb (gpointer data)
 		gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL, 
 				event_button->button, event_button->time);
 		return FALSE;
-	case 2:
+	case CA_FILE_ELEMENT_TYPE_CSR:
 		menu = glade_xml_get_widget (csr_popup_menu_xml,
 					     "csr_popup_menu");
 
@@ -1583,7 +1396,7 @@ gboolean ca_treeview_popup_timeout_program_cb (gpointer data)
 				event_button->button, event_button->time);
 		return FALSE;
 	default:
-	case 0:
+	case -1:
 		return FALSE;
 	}
 
@@ -1714,7 +1527,7 @@ void ca_on_change_pwd_menuitem_activate (GtkMenuItem *menuitem, gpointer user_da
 		repeat = (ca_file_is_password_protected() && ! ca_file_check_password (currpwd));
 		
 		if (repeat) {
-			ca_error_dialog (_("The current password you have entered  "
+			dialog_error (_("The current password you have entered  "
 					   "doesn't match with the actual current database password."));
 			widget = glade_xml_get_widget (dialog_xml, "ca_changepwd_current_pwd_entry");
 			gtk_widget_grab_focus (widget);
@@ -1729,7 +1542,7 @@ void ca_on_change_pwd_menuitem_activate (GtkMenuItem *menuitem, gpointer user_da
 			// It's a password change
 
 			if (! ca_file_password_change (currpwd, newpwd)) {
-				ca_error_dialog (_("Error while changing database password. "
+				dialog_error (_("Error while changing database password. "
 						   "The operation was cancelled."));
 			} else {
 				widget = glade_xml_get_widget (dialog_xml, "change_password_dialog");
@@ -1748,7 +1561,7 @@ void ca_on_change_pwd_menuitem_activate (GtkMenuItem *menuitem, gpointer user_da
 			// It's a new password
 
 			if (! ca_file_password_protect (newpwd)) {
-				ca_error_dialog (_("Error while establishing database password. "
+				dialog_error (_("Error while establishing database password. "
 						   "The operation was cancelled."));
 
 			} else {
@@ -1768,7 +1581,7 @@ void ca_on_change_pwd_menuitem_activate (GtkMenuItem *menuitem, gpointer user_da
 		if (ca_file_is_password_protected()) {
 			// Remove password protection
 			if (! ca_file_password_unprotect (currpwd)) {
-				ca_error_dialog (_("Error while removing database password. "
+				dialog_error (_("Error while removing database password. "
 						   "The operation was cancelled."));
 			} else {
 				widget = glade_xml_get_widget (dialog_xml, "change_password_dialog");
@@ -1927,12 +1740,12 @@ void ca_generate_dh_param_show (GtkWidget *menuitem, gpointer user_data)
 
 		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog2));
 
-		strerror = ca_generate_dh_param (dh_size, filename);
+		strerror = export_dh_param (dh_size, filename);
 
 		if (strerror) {
 			gtk_widget_destroy (GTK_WIDGET(dialog2));
 			gtk_widget_destroy (GTK_WIDGET(dialog));
-			ca_error_dialog (strerror);
+			dialog_error (strerror);
 		} else {
 			gtk_widget_destroy (GTK_WIDGET(dialog2));
 			gtk_widget_destroy (GTK_WIDGET(dialog));
@@ -1954,107 +1767,151 @@ void ca_generate_dh_param_show (GtkWidget *menuitem, gpointer user_data)
 	return;
 }
 
+
+/*
+ *
+ *   FILE MENU CALLBACKS
+ *
+ */ 
+
+
+void on_add_self_signed_ca_activate  (GtkMenuItem *menuitem, gpointer     user_data)
+{
+	new_ca_window_display();
+	
+}
+
+void on_add_csr_activate  (GtkMenuItem *menuitem, gpointer     user_data)
+{
+	new_req_window_display();
+	
+}
+
+
+void on_import1_activate  (GtkMenuItem *menuitem, gpointer     user_data)
+{
+
+	gchar *filename;
+
+	GtkWidget *dialog, *main_window_widget, *widget;
+	GladeXML * dialog_xml = NULL;
+        GtkToggleButton *radiobutton = NULL;
+	gchar     * xml_file = NULL;
+	gint response = 0;
+        gboolean import_file = TRUE;
+	
+	main_window_widget = glade_xml_get_widget (main_window_xml, "main_window");
+
+	xml_file = g_build_filename (PACKAGE_DATA_DIR, "gnomint", "gnomint.glade", NULL );
+	dialog_xml = glade_xml_new (xml_file, "import_file_or_directory_dialog", NULL);
+	g_free (xml_file);
+	glade_xml_signal_autoconnect (dialog_xml); 	
+
+        widget = glade_xml_get_widget (dialog_xml, "import_file_or_directory_dialog");
+        response = gtk_dialog_run (GTK_DIALOG(widget));
+
+        if (response < 0) {
+                gtk_widget_destroy (widget);
+                g_object_unref (G_OBJECT(dialog_xml));
+                return;
+        }
+
+        radiobutton = GTK_TOGGLE_BUTTON(glade_xml_get_widget (dialog_xml, "importfile_radiobutton"));
+        import_file = gtk_toggle_button_get_active(radiobutton);
+
+        gtk_widget_destroy (widget);
+
+        if (import_file) {
+                // Import single file
+                dialog = gtk_file_chooser_dialog_new (_("Select PEM file to import"),
+                                                      GTK_WINDOW(main_window_widget),
+                                                      GTK_FILE_CHOOSER_ACTION_OPEN,
+                                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                                      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                                                      NULL);
+                
+                if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+                {
+                        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+                        gtk_widget_destroy (dialog);
+                } else {
+                        gtk_widget_destroy (dialog);
+                        return;
+                }		
+                
+                if (! import_single_file (filename, NULL, NULL)) {
+                        dialog = gtk_message_dialog_new (GTK_WINDOW(main_window_widget),
+                                                         GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                         GTK_MESSAGE_ERROR,
+                                                         GTK_BUTTONS_CLOSE,
+                                                         _("Problem when importing '%s' file"),
+                                                         filename);
+                        
+                        gtk_dialog_run (GTK_DIALOG(dialog));
+                        
+                        gtk_widget_destroy (dialog);
+                }
+                return;
+        } else {
+                // Import directory
+
+                gchar * result = NULL;
+
+                dialog = gtk_file_chooser_dialog_new (_("Select directory to import"),
+                                                      GTK_WINDOW(main_window_widget),
+                                                      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+                                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                                      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                                                      NULL);
+                
+                if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+                {
+                        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+                        gtk_widget_destroy (dialog);
+                } else {
+                        gtk_widget_destroy (dialog);
+                        return;
+                }		
+
+                result = import_whole_dir (filename);
+
+                if (result) {
+                        dialog = gtk_message_dialog_new (GTK_WINDOW(main_window_widget),
+                                                         GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                         GTK_MESSAGE_ERROR,
+                                                         GTK_BUTTONS_CLOSE,
+                                                         "%s", result);
+                        
+                        gtk_dialog_run (GTK_DIALOG(dialog));
+                        
+                        gtk_widget_destroy (dialog);
+                }
+                return;
+
+        }
+}
+
+
+/*
+ *
+ *   EDIT MENU CALLBACKS
+ *
+ */ 
+
+
+
+void on_preferences1_activate  (GtkMenuItem *menuitem, gpointer     user_data)
+{
+        preferences_window_display ();
+}
+
+void on_properties1_activate  (GtkMenuItem *menuitem, gpointer     user_data)
+{
+	ca_treeview_row_activated (NULL, NULL, NULL, NULL);
+}
+
+
+
+
 #endif
 
-gchar *ca_generate_dh_param (guint dh_size, gchar *filename)
-{
-	GIOChannel * file = NULL;
-	gchar *pem = NULL;
-	GError * error = NULL;
-
-	pem = tls_generate_dh_params (dh_size);
-
-	file = g_io_channel_new_file (filename, "w", &error);
-	if (error) {
-		return (_("There was an error while saving Diffie-Hellman parameters."));
-	} 
-
-	g_io_channel_write_chars (file, pem, strlen(pem), NULL, &error);
-	if (error) {
-		return (_("There was an error while saving Diffie-Hellman parameters."));
-	} 
-
-	g_io_channel_shutdown (file, TRUE, &error);
-	if (error) {
-		return (_("There was an error while saving Diffie-Hellman parameters."));
-	} 
-
-	g_io_channel_unref (file);
-
-	return NULL;
-}
-
-
-gchar * ca_export_private_pkcs8 (guint64 id, gint type, gchar *filename)
-{
-	GIOChannel * file = NULL;
-	gchar * password = NULL;
-	GError * error = NULL;
-	gchar * dn = NULL;
-	PkeyManageData * crypted_pkey = NULL;
-	gchar * privatekey = NULL;
-	gchar * pem = NULL;
-
-
-	file = g_io_channel_new_file (filename, "w", &error);
-	if (error) {
-		g_free (password);
-		return (_("There was an error while exporting private key."));
-	} 
-	
-	crypted_pkey = pkey_manage_get_certificate_pkey (id);
-	dn = ca_file_get_dn_from_id (type, id);
-			
-	if (!crypted_pkey || !dn) {
-		pkey_manage_data_free (crypted_pkey);
-		g_free (dn);
-		return (_("There was an error while getting private key."));
-	}
-
-	privatekey = pkey_manage_uncrypt (crypted_pkey, dn);
-	
-	pkey_manage_data_free (crypted_pkey);
-	g_free (dn);
-
-	if (! privatekey) {
-		return (_("There was an error while uncrypting private key."));
-	}
-	
-	password = ca_dialog_get_password (_("You need to supply a passphrase for protecting the exported private key, "
-					     "so nobody else but authorized people can use it. This passphrase will be asked "
-					     "by any application that will make use of the private key."),
-					   _("Insert passphrase (8 characters or more):"), _("Insert passphrase (confirm):"), 
-					   _("The introduced passphrases are distinct."), 8);
-	if (! password) {
-		g_free (privatekey);
-		return (_("Operation cancelled."));
-	}
-
-	pem = tls_generate_pkcs8_encrypted_private_key (privatekey, password); 
-	g_free (password);
-	g_free (privatekey);
-	
-	if (!pem) {
-		return (_("There was an error while password-protecting private key."));
-	}
-	
-	g_io_channel_write_chars (file, pem, strlen(pem), NULL, &error);
-	if (error) {
-		g_free (pem);
-		return (_("There was an error while exporting private key."));
-	} 
-	g_free (pem);
-	
-	
-	g_io_channel_shutdown (file, TRUE, &error);
-	if (error) {
-		g_io_channel_unref (file);
-		return (_("There was an error while exporting private key."));
-	} 
-	
-	g_io_channel_unref (file);
-	
-	ca_file_mark_pkey_as_extracted_for_id (type, filename, id);
-
-	return NULL;
-}
