@@ -1,5 +1,5 @@
 //  gnoMint: a graphical interface for managing a certification authority
-//  Copyright (C) 2006,2007,2008 David Marín Carreño <davefx@gmail.com>
+//  Copyright (C) 2006-2009 David Marín Carreño <davefx@gmail.com>
 //
 //  This file is part of gnoMint.
 //
@@ -28,7 +28,7 @@
 
 #include "import.h"
 #include "tls.h"
-#include "ca.h"
+#include "dialog.h"
 #include "ca_file.h"
 
 
@@ -85,7 +85,7 @@ gchar * __import_ask_password (const gchar *crypted_part_description)
 		  "appropiate password.\n"));
 
 	prompt = g_strdup_printf (_("Please introduce password for `%s'"), crypted_part_description);
-	password = ca_ask_for_password (prompt);
+	password = dialog_ask_for_password (prompt);
 	g_free (prompt);
 	
 	return password;
@@ -95,23 +95,12 @@ gchar * __import_ask_password (const gchar *crypted_part_description)
 
 gint __import_csr (gnutls_x509_crq_t *crq, gchar ** csr_dn, guint64 *id)
 {
-	CaCreationData * creation_data = g_new0(CaCreationData, 1);
 	gchar * pem_csr=NULL;
 	size_t size;
 	gchar * error_msg;
         gint result = -1;
 	gchar *aux = NULL;
 	
-	size = 0;
-	gnutls_x509_crq_get_dn_by_oid (*crq, GNUTLS_OID_X520_COMMON_NAME, 0, 0, aux, &size);
-	if (size) {
-		aux = g_new0(gchar, size);
-		gnutls_x509_crq_get_dn_by_oid (*crq, GNUTLS_OID_X520_COMMON_NAME, 0, 0, aux, &size);
-		creation_data->cn = g_strdup (aux);
-		g_free (aux);
-		aux = NULL;
-	}	        
-	
         
         if (csr_dn) {
                 size = 0;
@@ -133,14 +122,14 @@ gint __import_csr (gnutls_x509_crq_t *crq, gchar ** csr_dn, guint64 *id)
 		
 	}
 	
-	error_msg = ca_file_insert_csr (creation_data, NULL, pem_csr, id);
+	error_msg = ca_file_insert_csr (NULL, pem_csr, NULL, id);
 	
 	
 	if (error_msg) {
 		gchar *message = g_strdup_printf (_("Couldn't import the certificate request. \n"
 						    "The database returned this error: \n\n'%s'"),
 						  error_msg);
-		ca_error_dialog (message);
+		dialog_error (message);
 		g_free (message);
 	} else {
                 result = 1;
@@ -150,7 +139,6 @@ gint __import_csr (gnutls_x509_crq_t *crq, gchar ** csr_dn, guint64 *id)
 
 gint __import_cert (gnutls_x509_crt_t *cert, gchar **cert_dn, guint64 *id)
 {
-	CertCreationData * creation_data = g_new0(CertCreationData, 1);
 	guchar *serial_str = NULL;
 	UInt160 serial;
 	gchar * pem_cert=NULL;
@@ -197,13 +185,6 @@ gint __import_cert (gnutls_x509_crt_t *cert, gchar **cert_dn, guint64 *id)
         }
 
 	
-	// Activation
-	creation_data->activation = gnutls_x509_crt_get_activation_time (*cert);
-	
-	// Expiration
-	creation_data->expiration = gnutls_x509_crt_get_expiration_time (*cert);
-        
-	
 	// Now we re-export the PEM (as the original PEM can be a list of certs)
 	size = 0;
 	gnutls_x509_crt_export (*cert, GNUTLS_X509_FMT_PEM, aux, &size);
@@ -215,7 +196,7 @@ gint __import_cert (gnutls_x509_crt_t *cert, gchar **cert_dn, guint64 *id)
 		aux = NULL;
 	}
 	
-	error_msg = ca_file_insert_imported_cert (creation_data, is_ca, serial, pem_cert, id);
+	error_msg = ca_file_insert_imported_cert (is_ca, serial, pem_cert, id);
         
 	if (pem_cert)
 		g_free (pem_cert);
@@ -225,7 +206,7 @@ gint __import_cert (gnutls_x509_crt_t *cert, gchar **cert_dn, guint64 *id)
 		gchar *message = g_strdup_printf (_("Couldn't import the certificate. \n"
 						    "The database returned this error: \n\n'%s'"),
 						  error_msg);
-		ca_error_dialog (message);
+		dialog_error (message);
 		g_free (message);
 	} else {
                 result = 1;
@@ -422,9 +403,9 @@ gint import_pkey_wo_passwd (guchar *file_contents, gsize file_contents_size)
 			
 		}
 
-                result_import = ca_file_import_privkey (pem_privkey);
+                result_import = ca_file_insert_imported_privkey (pem_privkey);
 		if (result_import) {
-                        ca_error_dialog (result_import);
+                        dialog_error (result_import);
                 } else {
                         result = 1;
                 }
@@ -557,9 +538,9 @@ gint import_pkcs8 (guchar *file_contents, gsize file_contents_size)
 			
 		}
 
-                error_msg = ca_file_import_privkey (pem_privkey);
+                error_msg = ca_file_insert_imported_privkey (pem_privkey);
 		if (error_msg) {
-                        ca_error_dialog (error_msg);
+                        dialog_error (error_msg);
                 } else {
                         result = 1;
                 }
@@ -589,7 +570,7 @@ gint import_pkcs8 (guchar *file_contents, gsize file_contents_size)
                         g_free (password);
 
                         if (result_decryption == GNUTLS_E_DECRYPTION_FAILED) {
-                                ca_error_dialog (_("The given password doesn't match the one used for crypting this part"));
+                                dialog_error (_("The given password doesn't match the one used for crypting this part"));
                         }
                 }
 
@@ -608,9 +589,9 @@ gint import_pkcs8 (guchar *file_contents, gsize file_contents_size)
                                 
                         }
                         
-                        error_msg = ca_file_import_privkey (pem_privkey);
+                        error_msg = ca_file_insert_imported_privkey (pem_privkey);
                         if (error_msg) {
-                                ca_error_dialog (error_msg);
+                                dialog_error (error_msg);
                         } else {
                                 result = 1;
                         }
@@ -700,7 +681,7 @@ gint import_pkcs12 (guchar *file_contents, gsize file_contents_size)
                                                                                      password));
                                 if (!pkcs12_bag_decrypted) {                                                
                                         gint j;
-                                        ca_error_dialog (_("The given password doesn't match with the password used for encrypting this part."));
+                                        dialog_error (_("The given password doesn't match with the password used for encrypting this part."));
                                         for (j=0; j<n_bags; j++) {
                                                 gnutls_pkcs12_bag_deinit (* g_array_index (pkcs_bag_array, gnutls_pkcs12_bag_t *, j));
                                                 g_free (g_array_index (pkcs_bag_array, gnutls_pkcs12_bag_t *, j));
@@ -774,7 +755,7 @@ gint import_pkcs12 (guchar *file_contents, gsize file_contents_size)
                                                                                                               password, 0);
 
                                                         if (result_decryption == GNUTLS_E_DECRYPTION_FAILED) {
-                                                                ca_error_dialog (_("The given password doesn't match the one used "
+                                                                dialog_error (_("The given password doesn't match the one used "
                                                                                    "for crypting this part"));
                                                         }
                                                 }
@@ -800,9 +781,9 @@ gint import_pkcs12 (guchar *file_contents, gsize file_contents_size)
                                                         
                                                 }
                                                 
-                                                error_msg = ca_file_import_privkey (pem_privkey);
+                                                error_msg = ca_file_insert_imported_privkey (pem_privkey);
                                                 if (error_msg) {
-                                                        ca_error_dialog (error_msg);
+                                                        dialog_error (error_msg);
                                                 } else {
                                                         result = 1;
                                                 }
@@ -863,7 +844,7 @@ gboolean import_single_file (gchar *filename, gchar **dn, guint64 *id)
 	GMappedFile * mapped_file = g_mapped_file_new (filename, FALSE, &error);
 
 	if (error) {
-		ca_error_dialog (_(error->message));
+		dialog_error (_(error->message));
 		return FALSE;
 	}
 
@@ -909,9 +890,9 @@ gboolean import_single_file (gchar *filename, gchar **dn, guint64 *id)
         g_free (file_contents);
 
 	if (successful_import) {
-		ca_refresh_model();
+		dialog_refresh_list();
 	} else {
-		ca_error_dialog (_("Couldn't find any supported format in the given file"));
+		dialog_error (_("Couldn't find any supported format in the given file"));
 	}
 
 	return TRUE;
@@ -925,7 +906,7 @@ gint import_openssl_private_key (const gchar *filename, gchar **last_password, g
 
 	if (! g_file_get_contents (filename, &filecontents, NULL, NULL)) {
 		gchar *message = g_strdup_printf(_("Couldn't open %s file. Check permissions."), filename);
-		ca_error_dialog (message);
+		dialog_error (message);
 		g_free (message);
 		return result;
 	}
@@ -952,7 +933,7 @@ gint import_openssl_private_key (const gchar *filename, gchar **last_password, g
 
 		if (!keytype) {
 			gchar * message = g_strdup_printf(_("Couldn't recognize the file %s as a RSA or DSA private key."), filename);
-			ca_error_dialog (message);
+			dialog_error (message);
 			g_free (message);
 			g_free (filecontents);
 			return result;
@@ -1004,7 +985,7 @@ gint import_openssl_private_key (const gchar *filename, gchar **last_password, g
 				g_free (filecontents);
 				g_free (temp_pwd);
 				g_free (dirname);
-				ca_error_dialog (_("Problem while calling to openssl for decyphering private key."));
+				dialog_error (_("Problem while calling to openssl for decyphering private key."));
 				break;					
 			}
 				
@@ -1015,7 +996,7 @@ gint import_openssl_private_key (const gchar *filename, gchar **last_password, g
 				gchar *error_to_show = g_strdup_printf (_("OpenSSL has returned the following error "
 									  "while trying to decypher the private key:\n\n%s"),
 									error_message);
-				ca_error_dialog (error_to_show);
+				dialog_error (error_to_show);
 				g_free (error_to_show);
 				first_time = FALSE;
 			}
@@ -1036,7 +1017,7 @@ gint import_openssl_private_key (const gchar *filename, gchar **last_password, g
 	result = import_pkey_wo_passwd ((guchar *) filecontents, strlen(filecontents));
 	
 	if (result == 1)
-		ca_refresh_model();
+		dialog_refresh_list();
 
 	g_free (filecontents);
 
@@ -1067,13 +1048,8 @@ gchar * import_whole_dir (gchar *dirname)
 
         // First, we try to probe if this is really a CA-containing directory
         
-        // * Try to detect OpenSSL CA
+        // * Try to detect OpenSSL CA.pl or TinyCA
 	{
-		filename = g_build_filename (dirname, "cacert.key", NULL);
-		if (! g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
-			error = TRUE;
-		}
-		g_free (filename);
 		filename = g_build_filename (dirname, "cacert.pem", NULL);
 		if (! g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
 			error = TRUE;
@@ -1094,19 +1070,25 @@ gchar * import_whole_dir (gchar *dirname)
 			error = TRUE;
 		}
 		g_free (filename);
-		filename = g_build_filename (dirname, "keys", NULL);
-		if (! g_file_test(filename, G_FILE_TEST_IS_DIR)) {
-			error = TRUE;
-		}
-		g_free (filename);
-		filename = g_build_filename (dirname, "req", NULL);
-		if (! g_file_test(filename, G_FILE_TEST_IS_DIR)) {
-			error = TRUE;
+		filename = g_build_filename (dirname, "cacert.key", NULL);
+		if (! g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
+			g_free (filename);
+			filename = g_build_filename (dirname, "private", "cakey.pem", NULL);
+			if (! g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
+				error = TRUE;
+			} else {
+				if (! error) 
+					CA_directory_type = 2; // OpenSSL CA.pl
+			}
 		}
 		g_free (filename);
 
-		if (! error) {
-			CA_directory_type = 1; //OpenSSL
+		if (! error && ! CA_directory_type) {
+			CA_directory_type = 1; //TinyCA
+		} 
+		
+		if (error) {
+			CA_directory_type = 0;
 		}
 	}
 
@@ -1114,6 +1096,7 @@ gchar * import_whole_dir (gchar *dirname)
 
         switch (CA_directory_type) {
 	case 1:
+	case 2:
                 
                 descriptions = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
@@ -1129,7 +1112,12 @@ gchar * import_whole_dir (gchar *dirname)
 		// Now we import the private CA root-certificate private key
 		// Usually, it is crypted in a OpenSSL proprietary format.
 		// Let's check it:
-		filename = g_build_filename (dirname, "cacert.key", NULL);
+		if (CA_directory_type == 1 /* TinyCA */) {
+			filename = g_build_filename (dirname, "cacert.key", NULL);
+		} else {
+			/* CA_directory_type == 2  (OpenSSL's CA.pl) */
+			filename = g_build_filename (dirname, "private", "cakey.pem", NULL);
+		}
 		if (import_openssl_private_key (filename, &ca_password, _("CA Root certificate")) == 0) {
 			g_free (filename);
 			result = _("There was a problem while importing the private key corresponding to CA root certificate");
@@ -1143,57 +1131,83 @@ gchar * import_whole_dir (gchar *dirname)
 		if (! dir) {
 			g_free (filename);
 			result = _("There was a problem while opening the directory certs/.");
-			break;
+		} else {
+			g_free (filename);
+			while ((int_filename = g_dir_read_name (dir))) {
+				
+				if (g_strrstr (int_filename, ".pem")) {
+					gchar *description = NULL;
+					filename = g_build_filename (dirname, "certs", int_filename, NULL);
+					if (import_single_file ((gchar *) filename, &description, NULL) == 0) {
+						problematic_files = g_list_append (problematic_files, g_strdup(filename));
+					} else {
+						if (description && ! g_hash_table_lookup (descriptions, int_filename))
+							g_hash_table_insert (descriptions, g_strdup(int_filename), description);
+					}
+					g_free (filename);
+				}
+			}
+			g_dir_close (dir);
 		}
-		g_free (filename);
-		while ((int_filename = g_dir_read_name (dir))) {
-			
-			if (g_strrstr (int_filename, ".pem")) {
-                                gchar *description = NULL;
-				filename = g_build_filename (dirname, "certs", int_filename, NULL);
-				if (import_single_file ((gchar *) filename, &description, NULL) == 0) {
-					problematic_files = g_list_append (problematic_files, g_strdup(filename));
-				} else {
-                                        if (description && ! g_hash_table_lookup (descriptions, int_filename))
-                                                g_hash_table_insert (descriptions, g_strdup(int_filename), description);
-                                }
+		filename = g_build_filename (dirname, "newcerts", NULL);
+		if (g_file_test(filename, G_FILE_TEST_IS_DIR)) { 
+			dir = g_dir_open (filename, 0, &gerror);
+			if (! dir) {
 				g_free (filename);
+				result = _("There was a problem while opening the directory newcerts/.");
+			} else {
+				g_free (filename);
+				while ((int_filename = g_dir_read_name (dir))) {
+					
+					if (g_strrstr (int_filename, ".pem")) {
+						gchar *description = NULL;
+						filename = g_build_filename (dirname, "newcerts", int_filename, NULL);
+						if (import_single_file ((gchar *) filename, &description, NULL) == 0) {
+							problematic_files = g_list_append (problematic_files, g_strdup(filename));
+						} else {
+							if (description && ! g_hash_table_lookup (descriptions, int_filename))
+								g_hash_table_insert (descriptions, g_strdup(int_filename), description);
+						}
+						g_free (filename);
+					}
+				}
+				g_dir_close (dir);
 			}
 		}
-		g_dir_close (dir);
-
 
 		// Now we import all the CSRs of the CA
 		filename = g_build_filename (dirname, "req", NULL);
-		dir = g_dir_open (filename, 0, &gerror);
-		if (! dir) {
-			g_free (filename);
-			result = _("There was a problem while opening the directory certs/.");
-			break;
-		}
-		g_free (filename);
-		while ((int_filename = g_dir_read_name (dir))) {
-			
-			if (g_strrstr (int_filename, ".pem")) {
-                                gchar *description = NULL;
-				filename = g_build_filename (dirname, "req", int_filename, NULL);
-				if (import_single_file ((gchar *) filename, &description, NULL) == 0) {
-					problematic_files = g_list_append (problematic_files, g_strdup(filename));
-				} else {
-                                        if (description && ! g_hash_table_lookup (descriptions, int_filename))
-                                                g_hash_table_insert (descriptions, g_strdup(int_filename), description);
-                                }
+		if (g_file_test(filename, G_FILE_TEST_IS_DIR)) { 
+			dir = g_dir_open (filename, 0, &gerror);
+			if (! dir) {
 				g_free (filename);
+				result = _("There was a problem while opening the directory req.");
+				break;
 			}
+			g_free (filename);
+			while ((int_filename = g_dir_read_name (dir))) {
+				
+				if (g_strrstr (int_filename, ".pem")) {
+					gchar *description = NULL;
+					filename = g_build_filename (dirname, "req", int_filename, NULL);
+					if (import_single_file ((gchar *) filename, &description, NULL) == 0) {
+						problematic_files = g_list_append (problematic_files, g_strdup(filename));
+					} else {
+						if (description && ! g_hash_table_lookup (descriptions, int_filename))
+							g_hash_table_insert (descriptions, g_strdup(int_filename), description);
+					}
+					g_free (filename);
+				}
+			}
+			g_dir_close (dir);
 		}
-		g_dir_close (dir);
 
 		// Now we import all the CRLs of the CA
 		filename = g_build_filename (dirname, "crl", NULL);
 		dir = g_dir_open (filename, 0, &gerror);
 		if (! dir) {
 			g_free (filename);
-			result = _("There was a problem while opening the directory certs/.");
+			result = _("There was a problem while opening the directory crl/.");
 			break;
 		}
 		g_free (filename);
@@ -1211,38 +1225,39 @@ gchar * import_whole_dir (gchar *dirname)
 		
 		// Now we import all the private keys of the CA
 		filename = g_build_filename (dirname, "keys", NULL);
-		dir = g_dir_open (filename, 0, &gerror);
-		if (! dir) {
-			g_free (filename);
-			result = _("There was a problem while opening the directory certs/.");
-			break;
-		}
-		g_free (filename);
-		while ((int_filename = g_dir_read_name (dir))) {
-			
-			if (g_strrstr (int_filename, ".pem")) {
-                                gchar *description = NULL;
-
-				filename = g_build_filename (dirname, "keys", int_filename, NULL);
-
-                                description = g_hash_table_lookup (descriptions, int_filename);
-                                if (! description)
-                                        description = filename;
-
-				if (import_openssl_private_key (filename, &ca_password, description) == 0) {
-					problematic_files = g_list_append (problematic_files, g_strdup(filename));					
-				}
+		if (g_file_test(filename, G_FILE_TEST_IS_DIR)) { 
+			dir = g_dir_open (filename, 0, &gerror);
+			if (! dir) {
 				g_free (filename);
+				result = _("There was a problem while opening the directory keys/.");
+				break;
 			}
+			g_free (filename);
+			while ((int_filename = g_dir_read_name (dir))) {
+				
+				if (g_strrstr (int_filename, ".pem")) {
+					gchar *description = NULL;
+					
+					filename = g_build_filename (dirname, "keys", int_filename, NULL);
+					
+					description = g_hash_table_lookup (descriptions, int_filename);
+					if (! description)
+						description = filename;
+					
+					if (import_openssl_private_key (filename, &ca_password, description) == 0) {
+						problematic_files = g_list_append (problematic_files, g_strdup(filename));					
+					}
+					g_free (filename);
+				}
+			}
+			g_dir_close (dir);
 		}
-		g_dir_close (dir);
-                
 
 		// Now we import the last serial number
 		filename = g_build_filename (dirname, "serial", NULL);
                 if (! g_file_get_contents (filename, &filecontents, NULL, NULL)) {
                         gchar *message = g_strdup_printf(_("Couldn't open %s file. Check permissions."), filename);
-                        ca_error_dialog (message);
+                        dialog_error (message);
                         g_free (message);
                         return result;
                 }
