@@ -1,5 +1,5 @@
 //  gnoMint: a graphical interface for managing a certification authority
-//  Copyright (C) 2006,2007,2008 David Marín Carreño <davefx@gmail.com>
+//  Copyright (C) 2006-2009 David Marín Carreño <davefx@gmail.com>
 //
 //  This file is part of gnoMint.
 //
@@ -25,23 +25,14 @@
 #include <stdlib.h>
 
 #include "main.h"
-#include "new_ca_window.h"
-#include "new_req_window.h"
-#include "new_cert_window.h"
-#include "tls.h"
 #include "ca.h"
+#include "dialog.h"
+#include "tls.h"
 #include "ca_file.h"
 #include "preferences-gui.h"
-#include "preferences-window.h"
-#include "import.h"
 
 #define GNOMINT_MIME_TYPE "application/x-gnomint"
 
-/* gchar * PACKAGE_AUTHORS[] = { */
-/* 	"David Marín Carreño <davefx@gmail.com>", */
-/* 	NULL */
-/* }; */
-
 GladeXML * main_window_xml = NULL;
 GladeXML * csr_popup_menu_xml = NULL;
 GladeXML * cert_popup_menu_xml = NULL;
@@ -126,6 +117,10 @@ int main (int   argc,
 
 	g_thread_init (NULL);
 	gtk_init (&argc, &argv);
+	
+	preferences_gui_set_csr_visible_callback (ca_update_csr_view);
+	preferences_gui_set_revoked_visible_callback (ca_update_revoked_view);
+
         preferences_init (argc, argv);
 
 	ctx = g_option_context_new (_("- A graphical Certification Authority manager"));
@@ -182,6 +177,8 @@ int main (int   argc,
 	__disable_widget ("save_as1");
 	__disable_widget ("properties1");
 	__disable_widget ("preferences1");
+
+	dialog_establish_refresh_function (ca_refresh_model_callback);
 	
 
 	if (argc >= 2 && ca_open (g_strdup(argv[1]), TRUE)) {
@@ -221,24 +218,59 @@ gboolean on_main_window1_delete (GtkWidget *widget,
 	return TRUE;
 }
 
-
-/*
- *
- *   FILE MENU CALLBACKS
- *
- */ 
-
-
-void on_add_self_signed_ca_activate  (GtkMenuItem *menuitem, gpointer     user_data)
+void __recent_add_utf8_filename (const gchar *utf8_filename)
 {
-	new_ca_window_display();
-	
-}
+        GtkRecentData *recent_data;
+        gchar         *filename;
+        gchar         *uri;
+	gchar         *pwd;
+
+        static gchar *groups[2] = {
+                "gnomint",
+                NULL
+        };
+
+
+        recent_data = g_slice_new (GtkRecentData);
+
+        recent_data->display_name = NULL;
+        recent_data->description  = NULL;
+        recent_data->mime_type    = GNOMINT_MIME_TYPE;
+        recent_data->app_name     = (gchar *) g_get_application_name ();
+        recent_data->app_exec     = g_strjoin (" ", g_get_prgname (), "%f", NULL);
+        recent_data->groups       = groups;
+        recent_data->is_private = FALSE;
+
+        filename = g_filename_from_utf8 (utf8_filename, -1, NULL, NULL, NULL);
+        if ( filename != NULL )
+        {
+
+		if (! g_path_is_absolute (filename)) {
+			gchar *absolute_filename;
+
+			pwd = g_get_current_dir ();
+			absolute_filename = g_build_filename (pwd, filename, NULL);
+			g_free (pwd);
+			g_free (filename);
+			filename = absolute_filename;
+		}
+
+
+                uri = g_filename_to_uri (filename, NULL, NULL);
+                if ( uri != NULL )
+                {
+
+                        gtk_recent_manager_add_full (recent_manager, uri, recent_data);
+                        g_free (uri);
+
+                }
+                g_free (filename);
+
+        }
+
+        g_free (recent_data->app_exec);
+        g_slice_free (GtkRecentData, recent_data);
 
-void on_add_csr_activate  (GtkMenuItem *menuitem, gpointer     user_data)
-{
-	new_req_window_display();
-	
 }
 
 void on_new1_activate (GtkMenuItem *menuitem, gpointer     user_data)
@@ -307,63 +339,10 @@ void on_new1_activate (GtkMenuItem *menuitem, gpointer     user_data)
 		gtk_dialog_run (GTK_DIALOG(dialog));
 		
 		gtk_widget_destroy (dialog);
-	}
-	return;
-
-}
-
-void __recent_add_utf8_filename (const gchar *utf8_filename)
-{
-        GtkRecentData *recent_data;
-        gchar         *filename;
-        gchar         *uri;
-	gchar         *pwd;
-
-        static gchar *groups[2] = {
-                "gnomint",
-                NULL
-        };
-
-
-        recent_data = g_slice_new (GtkRecentData);
-
-        recent_data->display_name = NULL;
-        recent_data->description  = NULL;
-        recent_data->mime_type    = GNOMINT_MIME_TYPE;
-        recent_data->app_name     = (gchar *) g_get_application_name ();
-        recent_data->app_exec     = g_strjoin (" ", g_get_prgname (), "%f", NULL);
-        recent_data->groups       = groups;
-        recent_data->is_private = FALSE;
-
-        filename = g_filename_from_utf8 (utf8_filename, -1, NULL, NULL, NULL);
-        if ( filename != NULL )
-        {
-
-		if (! g_path_is_absolute (filename)) {
-			gchar *absolute_filename;
-
-			pwd = g_get_current_dir ();
-			absolute_filename = g_build_filename (pwd, filename, NULL);
-			g_free (pwd);
-			g_free (filename);
-			filename = absolute_filename;
-		}
-
-
-                uri = g_filename_to_uri (filename, NULL, NULL);
-                if ( uri != NULL )
-                {
-
-                        gtk_recent_manager_add_full (recent_manager, uri, recent_data);
-                        g_free (uri);
-
-                }
-                g_free (filename);
-
+	} else {
+		__recent_add_utf8_filename (filename);
         }
-
-        g_free (recent_data->app_exec);
-        g_slice_free (GtkRecentData, recent_data);
+	return;
 
 }
 
@@ -479,114 +458,12 @@ void on_save_as1_activate  (GtkMenuItem *menuitem, gpointer     user_data)
 		return;
 	}		
 	
-	ca_file_save_as (filename);
+	if (ca_file_save_as (filename))
+                __recent_add_utf8_filename (filename);
 
 }
 
 
-void on_import1_activate  (GtkMenuItem *menuitem, gpointer     user_data)
-{
-
-	gchar *filename;
-
-	GtkWidget *dialog, *main_window_widget, *widget;
-	GladeXML * dialog_xml = NULL;
-        GtkToggleButton *radiobutton = NULL;
-	gchar     * xml_file = NULL;
-	gint response = 0;
-        gboolean import_file = TRUE;
-	
-	main_window_widget = glade_xml_get_widget (main_window_xml, "main_window");
-
-	xml_file = g_build_filename (PACKAGE_DATA_DIR, "gnomint", "gnomint.glade", NULL );
-	dialog_xml = glade_xml_new (xml_file, "import_file_or_directory_dialog", NULL);
-	g_free (xml_file);
-	glade_xml_signal_autoconnect (dialog_xml); 	
-
-        widget = glade_xml_get_widget (dialog_xml, "import_file_or_directory_dialog");
-        response = gtk_dialog_run (GTK_DIALOG(widget));
-
-        if (response < 0) {
-                gtk_widget_destroy (widget);
-                g_object_unref (G_OBJECT(dialog_xml));
-                return;
-        }
-
-        radiobutton = GTK_TOGGLE_BUTTON(glade_xml_get_widget (dialog_xml, "importfile_radiobutton"));
-        import_file = gtk_toggle_button_get_active(radiobutton);
-
-        gtk_widget_destroy (widget);
-
-        if (import_file) {
-                // Import single file
-                dialog = gtk_file_chooser_dialog_new (_("Select PEM file to import"),
-                                                      GTK_WINDOW(main_window_widget),
-                                                      GTK_FILE_CHOOSER_ACTION_OPEN,
-                                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                                      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-                                                      NULL);
-                
-                if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
-                {
-                        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
-                        gtk_widget_destroy (dialog);
-                } else {
-                        gtk_widget_destroy (dialog);
-                        return;
-                }		
-                
-                if (! import_single_file (filename, NULL, NULL)) {
-                        dialog = gtk_message_dialog_new (GTK_WINDOW(main_window_widget),
-                                                         GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                         GTK_MESSAGE_ERROR,
-                                                         GTK_BUTTONS_CLOSE,
-                                                         _("Problem when importing '%s' file"),
-                                                         filename);
-                        
-                        gtk_dialog_run (GTK_DIALOG(dialog));
-                        
-                        gtk_widget_destroy (dialog);
-                }
-                return;
-        } else {
-                // Import directory
-
-                gchar * result = NULL;
-
-                dialog = gtk_file_chooser_dialog_new (_("Select directory to import"),
-                                                      GTK_WINDOW(main_window_widget),
-                                                      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
-                                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                                      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-                                                      NULL);
-                
-                if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
-                {
-                        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
-                        gtk_widget_destroy (dialog);
-                } else {
-                        gtk_widget_destroy (dialog);
-                        return;
-                }		
-
-                result = import_whole_dir (filename);
-
-                if (result) {
-                        dialog = gtk_message_dialog_new (GTK_WINDOW(main_window_widget),
-                                                         GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                         GTK_MESSAGE_ERROR,
-                                                         GTK_BUTTONS_CLOSE,
-                                                         "%s", result);
-                        
-                        gtk_dialog_run (GTK_DIALOG(dialog));
-                        
-                        gtk_widget_destroy (dialog);
-                }
-                return;
-
-        }
-}
-
 
 void on_quit1_activate  (GtkMenuItem *menuitem, gpointer     user_data)
 {
@@ -595,33 +472,6 @@ void on_quit1_activate  (GtkMenuItem *menuitem, gpointer     user_data)
 
 
 
-/*
- *
- *   EDIT MENU CALLBACKS
- *
- */ 
-
-void on_clear1_activate  (GtkMenuItem *menuitem, gpointer     user_data)
-{
-	printf ("clear1 Activated\n");
-}
-
-void on_properties1_activate  (GtkMenuItem *menuitem, gpointer     user_data)
-{
-	ca_treeview_row_activated (NULL, NULL, NULL, NULL);
-}
-
-void on_preferences1_activate  (GtkMenuItem *menuitem, gpointer     user_data)
-{
-        preferences_window_display ();
-}
-
-
-/*
- *
- *   VIEW MENU CALLBACKS
- *
- */ 
 
 
 /*
