@@ -1,5 +1,5 @@
 //  gnoMint: a graphical interface for managing a certification authority
-//  Copyright (C) 2006,2007,2008 David Marín Carreño <davefx@gmail.com>
+//  Copyright (C) 2006-2009 David Marín Carreño <davefx@gmail.com>
 //
 //  This file is part of gnoMint.
 //
@@ -24,27 +24,15 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "new_ca_window.h"
-#include "ca_creation.h"
-#include "ca_policy.h"
-#include "new_cert_creation_process.h"
+#include "creation_process_window.h"
 #include "ca_file.h"
+#include "country_table.h"
 #include "tls.h"
-#include "ca.h"
 #include "pkey_manage.h"
 #include "new_req_window.h"
 
 #include <glib/gi18n.h>
 
-typedef struct {
-	char * name;
-	char * code;
-} CountryItem;
-
-#define NUMBER_OF_COUNTRIES 244
-CountryItem country_table[NUMBER_OF_COUNTRIES];
-
-
 GladeXML * new_req_window_xml = NULL;
 GtkTreeStore * new_req_ca_list_model = NULL;
 gboolean new_req_ca_id_valid = FALSE;
@@ -241,7 +229,7 @@ void new_req_window_display()
 	
 	glade_xml_signal_autoconnect (new_req_window_xml); 	
 	
-	new_ca_populate_country_combobox(GTK_COMBO_BOX(glade_xml_get_widget(new_req_window_xml, "country_combobox1")));
+	country_table_populate_combobox(GTK_COMBO_BOX(glade_xml_get_widget(new_req_window_xml, "country_combobox1")));
 
 	__new_req_populate_ca_treeview (GTK_TREE_VIEW(glade_xml_get_widget(new_req_window_xml, "new_req_ca_treeview")));
 
@@ -413,7 +401,7 @@ void on_new_req_cancel_clicked (GtkButton *widget,
 void on_new_req_commit_clicked (GtkButton *widg,
 			       gpointer user_data) 
 {
-	CaCreationData *csr_creation_data = NULL;
+	TlsCreationData *csr_creation_data = NULL;
 
 	GtkWidget *widget = NULL;
 	GtkWindow *window = NULL;
@@ -422,7 +410,7 @@ void on_new_req_commit_clicked (GtkButton *widg,
 	GtkTreeModel *tree_model = NULL;
 	GtkTreeIter tree_iter;
 	
-	csr_creation_data = g_new0 (CaCreationData, 1);
+	csr_creation_data = g_new0 (TlsCreationData, 1);
 
         if (new_req_ca_id_valid)
                 csr_creation_data->parent_ca_id_str = g_strdup_printf ("'%"G_GUINT64_FORMAT"'", new_req_ca_id);
@@ -496,7 +484,7 @@ void on_new_req_commit_clicked (GtkButton *widg,
 	window = GTK_WINDOW(glade_xml_get_widget (new_req_window_xml, "new_req_window"));
 	gtk_object_destroy(GTK_OBJECT(window));
 
-	new_csr_creation_process_window_display (csr_creation_data);	
+	creation_process_window_csr_display (csr_creation_data);	
 
 }
 
