@@ -1,5 +1,5 @@
 //  gnoMint: a graphical interface for managing a certification authority
-//  Copyright (C) 2006,2007,2008 David Marín Carreño <davefx@gmail.com>
+//  Copyright (C) 2006-2009 David Marín Carreño <davefx@gmail.com>
 //
 //  This file is part of gnoMint.
 //
@@ -33,7 +33,7 @@
 
 #include "tls.h"
 #include "ca_file.h"
-#include "ca.h"
+#include "dialog.h"
 #include "pkey_manage.h"
 
 #include <glib/gi18n.h>
@@ -48,8 +48,11 @@ guchar * __pkey_manage_from_hex (const gchar *input);
 guchar *__pkey_manage_create_key(const gchar *password);
 gchar * __pkey_manage_aes_encrypt (const gchar *in, const gchar *password);
 gchar * __pkey_manage_aes_decrypt (const gchar *string, const gchar *password);
+gchar * __pkey_manage_aes_encrypt_aux (const gchar *in, const gchar *password, const guchar *iv, const guchar *ctr);
+gchar * __pkey_manage_aes_decrypt_aux (const gchar *string, const gchar *password, const guchar *iv, const guchar *ctr);
 
 #ifndef GNOMINTCLI
+
 // CALLBACKS
 gboolean pkey_manage_filechooser_file_set_cb (GtkFileChooserButton *widget, gpointer user_data);
 
@@ -165,7 +168,7 @@ gchar * __pkey_retrieve_from_file (gchar **fn, gchar *cert_pem)
 					
 					if (tls_error == TLS_INVALID_PASSWORD) {
 						if (password)
-							ca_error_dialog (_("The given password doesn't match with the one used while crypting the file."));
+							dialog_error (_("The given password doesn't match with the one used while crypting the file."));
 
 						// We ask for a password
 						password = __pkey_manage_ask_external_file_password (cert->dn);
@@ -181,21 +184,21 @@ gchar * __pkey_retrieve_from_file (gchar **fn, gchar *cert_pem)
 				if (! pem_pkey) {
 					if (tls_error == TLS_NON_MATCHING_PRIVATE_KEY) {
 						// The file could be opened, but it didn't contain any recognized private key
-						ca_error_dialog (_("The file designated in database contains a private key, but it "
+						dialog_error (_("The file designated in database contains a private key, but it "
 								   "is not the private key corresponding to the certificate."));
 					} else {
 						// The file could be opened, but it didn't contain any recognized private key
-						ca_error_dialog (_("The file designated in database doesn't contain any recognized private key."));
+						dialog_error (_("The file designated in database doesn't contain any recognized private key."));
 					}
 				}
 			} else {
 				// The file cannot be opened
-				ca_error_dialog (_("The file designated in database couldn't be opened."));
+				dialog_error (_("The file designated in database couldn't be opened."));
 				
 			}
 		} else {
 			// The file doesn't exist
-			ca_error_dialog (_("The file designated in database doesn't exist."));
+			dialog_error (_("The file designated in database doesn't exist."));
 			
 		}
 		
@@ -344,14 +347,14 @@ void pkey_manage_data_free (PkeyManageData *pkeydata)
 
 
 
-unsigned char iv[16] =
-    { 'a', 'g', 'z', 'e', 'Q', '5', 'E', '7', 'c', '+', '*', 'G', '1', 'D',
-	'u', '='
+guchar old_iv[16] =
+{ 'a', 'g', 'z', 'e', 'Q', '5', 'E', '7', 'c', '+', '*', 'G', '1', 'D',
+  'u', '='
 };
 
-unsigned char ctr[16] =
-    { 'd', 'g', '4', 'e', 'J', '5', '3', 'l', 'c', '-', '!', 'G', 'z', 'A',
-	'z', '='
+guchar old_ctr[16] =
+{ 'd', 'g', '4', 'e', 'J', '5', '3', 'l', 'c', '-', '!', 'G', 'z', 'A',
+  'z', '='
 };
 
 gchar *saved_password = NULL;
@@ -415,6 +418,49 @@ guchar *__pkey_manage_create_key(const gchar *password)
 }
 
 gchar * __pkey_manage_aes_encrypt (const gchar *in, const gchar *password)
+{
+	gchar * result = NULL;
+
+	gint i;
+	guint in_length = strlen (in);
+	guchar * random_data = NULL;
+	guchar * sha256 = NULL;
+	gchar * sha256_hex = NULL;
+
+	gchar * cyphered = NULL;
+
+	random_data = g_new0 (guchar, in_length);
+
+	// Fill random_data with random data
+	gcry_create_nonce (random_data, in_length);
+
+	// XOR random_data with in => random_data
+	for (i=0; i<in_length; i++) {
+		random_data[i] = random_data[i] ^ in[i];
+	}
+
+	// SHA-2 256 random_data
+	sha256 = g_new0 (guchar, gcry_md_get_algo_dlen(GCRY_MD_SHA256));
+	gcry_md_hash_buffer (GCRY_MD_SHA256, sha256, random_data, in_length);
+	
+	sha256_hex = __pkey_manage_to_hex (sha256, 32);
+
+	cyphered = __pkey_manage_aes_encrypt_aux (in, password, &sha256[0], &sha256[16]);
+	
+	result = g_strdup_printf ("gCP%s%s", sha256_hex, cyphered);
+
+	g_free (sha256_hex);
+	g_free (cyphered);
+
+	g_free (sha256);
+	g_free (random_data);
+
+	return result;
+	
+}
+
+
+gchar * __pkey_manage_aes_encrypt_aux (const gchar *in, const gchar *password, const guchar *iv, const guchar *ctr)
 {
 	guchar *key = __pkey_manage_create_key (password);
 	guchar *out = (guchar *) g_strdup(in);
@@ -429,17 +475,39 @@ gchar * __pkey_manage_aes_encrypt (const gchar *in, const gchar *password)
 		return NULL;
 	}
 
-	get = gcry_cipher_setiv(cry_ctxt, &iv, 16);
-	if (get) {
-		fprintf (stderr, "ERR GCRYPT: %ud\n", gcry_err_code(get));
-		return NULL;
-	}
-	get = gcry_cipher_setctr(cry_ctxt, &ctr, 16);
-	if (get) {
-		fprintf (stderr, "ERR GCRYPT: %ud\n", gcry_err_code(get));
-		return NULL;
-	}
+	if (! iv || !ctr) {
 
+		get = gcry_cipher_setiv(cry_ctxt, &old_iv, 16);
+		if (get) {
+			fprintf (stderr, "ERR GCRYPT: %ud\n", gcry_err_code(get));
+			return NULL;
+		}
+
+		if (! ctr)
+			ctr = old_ctr;
+
+		get = gcry_cipher_setctr(cry_ctxt, &old_ctr, 16);
+		if (get) {
+			fprintf (stderr, "ERR GCRYPT: %ud\n", gcry_err_code(get));
+			return NULL;
+		}
+
+	} else {
+		get = gcry_cipher_setiv(cry_ctxt, iv, 16);
+		if (get) {
+			fprintf (stderr, "ERR GCRYPT: %ud\n", gcry_err_code(get));
+			return NULL;
+		}
+
+		if (! ctr)
+			ctr = old_ctr;
+
+		get = gcry_cipher_setctr(cry_ctxt, ctr, 16);
+		if (get) {
+			fprintf (stderr, "ERR GCRYPT: %ud\n", gcry_err_code(get));
+			return NULL;
+		}
+	}
 	get = gcry_cipher_setkey (cry_ctxt, key, 32);
 	if (get) {
 		fprintf (stderr, "ERR GCRYPT: %ud\n", gcry_err_code(get));
@@ -462,6 +530,37 @@ gchar * __pkey_manage_aes_encrypt (const gchar *in, const gchar *password)
 }
 
 gchar * __pkey_manage_aes_decrypt (const gchar *string, const gchar *password)
+{
+	gchar iv_hex[33];
+	gchar ctr_hex[33];
+	const gchar *ciphered = NULL;
+	guchar *iv;
+	guchar *ctr;
+
+	gchar * result;
+	
+	// If it is not a new-type ciphered message, decrypt it with old static iv and ctr
+	if (strncmp (string, "gCP", 3)) {
+		return __pkey_manage_aes_decrypt_aux (string, password, NULL, NULL);
+	}
+
+	// It's a new-type ciphered message. 
+	strncpy (iv_hex, &string[3], 32);
+	strncpy (ctr_hex, &string[35], 32);
+	ciphered = &string[67];
+
+	iv = __pkey_manage_from_hex (iv_hex);
+	ctr = __pkey_manage_from_hex (ctr_hex);
+
+	result = __pkey_manage_aes_decrypt_aux (ciphered, password, iv, ctr);
+
+	g_free (iv);
+	g_free (ctr);
+	
+	return result;
+}
+
+gchar * __pkey_manage_aes_decrypt_aux (const gchar *string, const gchar *password, const guchar *iv, const guchar *ctr)
 {
 	guchar *out = __pkey_manage_from_hex(string);
 
@@ -476,15 +575,38 @@ gchar * __pkey_manage_aes_decrypt (const gchar *string, const gchar *password)
 		return NULL;
 	}
 
-	get = gcry_cipher_setiv(cry_ctxt, &iv, 16);
-	if (get) {
-		fprintf (stderr, "ERR GCRYPT: %ud\n", gcry_err_code(get));
-		return NULL;
-	}
-	get = gcry_cipher_setctr(cry_ctxt, &ctr, 16);
-	if (get) {
-		fprintf (stderr, "ERR GCRYPT: %ud\n", gcry_err_code(get));
-		return NULL;
+	if (! iv || !ctr) {
+
+		get = gcry_cipher_setiv(cry_ctxt, &old_iv, 16);
+		if (get) {
+			fprintf (stderr, "ERR GCRYPT: %ud\n", gcry_err_code(get));
+			return NULL;
+		}
+
+		if (! ctr)
+			ctr = old_ctr;
+
+		get = gcry_cipher_setctr(cry_ctxt, &old_ctr, 16);
+		if (get) {
+			fprintf (stderr, "ERR GCRYPT: %ud\n", gcry_err_code(get));
+			return NULL;
+		}
+
+	} else {
+		get = gcry_cipher_setiv(cry_ctxt, iv, 16);
+		if (get) {
+			fprintf (stderr, "ERR GCRYPT: %ud\n", gcry_err_code(get));
+			return NULL;
+		}
+
+		if (! ctr)
+			ctr = old_ctr;
+
+		get = gcry_cipher_setctr(cry_ctxt, ctr, 16);
+		if (get) {
+			fprintf (stderr, "ERR GCRYPT: %ud\n", gcry_err_code(get));
+			return NULL;
+		}
 	}
 
 	get = gcry_cipher_setkey (cry_ctxt, key, 32);
@@ -540,9 +662,19 @@ gboolean pkey_manage_check_password (const gchar *checking_password, const gchar
 
 	password = g_strdup_printf ("%sgnoMintPassword%s", salt, checking_password);
 
-	cp = __pkey_manage_aes_encrypt (password, password);
+	if (strncmp (&hashed_password[2], "gCP", 3)) {
+		cp = __pkey_manage_aes_encrypt_aux (password, password, NULL, NULL);
+		res = (! strcmp(cp, &hashed_password[2]));
+		
+		g_free (cp);
+		g_free (password);
+		
+		return res;		
+	} 
 
-	res = (! strcmp(cp, &hashed_password[2]));
+	cp = __pkey_manage_aes_decrypt (&hashed_password[2], password);
+
+	res = (! strcmp(cp, password));
 
 	g_free (cp);
 	g_free (password);
@@ -550,7 +682,7 @@ gboolean pkey_manage_check_password (const gchar *checking_password, const gchar
 	return res;
 }
 
-void pkey_manage_crypt_auto (CaCreationData *creation_data,
+void pkey_manage_crypt_auto (gchar *password,
 			     gchar **pem_private_key,
 			     const gchar *pem_ca_certificate)
 {
@@ -560,7 +692,7 @@ void pkey_manage_crypt_auto (CaCreationData *creation_data,
 
 	tls_cert = tls_parse_cert_pem (pem_ca_certificate);
 
-	res = pkey_manage_crypt_w_pwd (clean_private_key, tls_cert->dn, creation_data->password);
+	res = pkey_manage_crypt_w_pwd (clean_private_key, tls_cert->dn, password);
 
 	tls_cert_free (tls_cert);
 
@@ -625,7 +757,7 @@ gchar * pkey_manage_ask_password ()
 		is_key_ok = ca_file_check_password (password);
 		
 		if (! is_key_ok) {
-			ca_error_dialog (_("The given password doesn't match the one used in the database"));
+			dialog_error (_("The given password doesn't match the one used in the database"));
 		}
 
 	}
@@ -676,7 +808,7 @@ gchar * pkey_manage_ask_password ()
 		is_key_ok = ca_file_check_password (password);
 		
 		if (! is_key_ok) {
-			ca_error_dialog (_("The given password doesn't match the one used in the database"));
+			dialog_error (_("The given password doesn't match the one used in the database"));
 		}
 
 	}
